#ifndef ANALYSE_H
#define ANALYSE_H

#include "figures.h"
#include <QList>
#include <QStringList>
#include <QtDebug>

class ResultatAnalyse
{
	// le résultat de l'analyse.
	// action : soit on relance certains dés, soit on marque une ligne

// 	Q_OBJECT
			
	public:
        ResultatAnalyse();
        ~ResultatAnalyse() {};
		
		enum { relancer, marquerLigne };
		
		void clear();
		
        int getAction() { return m_action; }
		void setAction( int ligne, int colonne );
		void setAction( QList<int> liste );
		
        QList<int> getRejouer() { return m_rejouer; }
        void setRejouer( QList<int> aRejouer ) { m_rejouer = aRejouer; }
		
		int getLigne() { return m_ligne; }
		void setLigne ( int ligne ) { m_ligne = ligne; }
		int getColonne() { return m_colonne; }
		void setColonne ( int colonne ) { m_colonne = colonne; }

		bool isDirty() { return m_action == relancer && m_rejouer.isEmpty(); }

		void trace(QString);
		QStringList getTrace() { return m_trace; }
		QString getLabelLigne (int ligne) { return m_labelLigne[ligne]; }
		QString getLabelColonne (int colonne) { return m_labelColonne[colonne]; }
	
	private:
		int m_action;
		QList<int> m_rejouer;
		int m_ligne;
		int m_colonne;
		QStringList m_trace;
		QStringList m_labelLigne;
		QStringList m_labelColonne;
};


class Figure
{
	public:
        Figure() { clear(); }
        ~Figure() {}
		void clear();
        void set( QList<int> list, bool pure, int val = -1 ) { listeDes = list; pur = pure; valeur = val; }
        bool exists() { return !listeDes.isEmpty(); }
        bool isPure() { return pur;}
        void setPure(bool pure) { pur = pure; }
        int getValue() { return valeur; }
        QList<int> getList() { return listeDes; }
		Figure operator=(const Figure autre);
		Figure operator+(const Figure autre);
		bool contains(int numDice) { return listeDes.contains(numDice); };
		

	private:
		int valeur;
		bool pur;
		QList<int> listeDes;
};


class Analyse : public QObject
{
// objet de la classe : décompte des points et extraction des figures pour préparer l'Analyse proprement dite


	Q_OBJECT

public:
	Analyse ();
    ~Analyse () {}
	void init(int joueur, int numSession, int ligneDescendante, int ligneMontante, QList<int> &lignesLibres, int d1, int d2, int d3, int d4, int d5, int ecartDesc, int ecartAsc, int ecartLibre);
	void setSuggestions();
	QList<int> getPossible();
    ResultatAnalyse getResultatAnalyse() { return ra; }
	QString getLabel( int ligne) { return label[ligne]; }
	
	int valeurFigure( int figure );
    int getCount( int figure ){ return (figure >= UN && figure <= SIX) ? (resultat[figure][NBR]) : -1;}
    int getPoints( int figure ) { return resultat[figure][POINTS]; }
    void setPoints( int figure, int pts ) { resultat[figure][POINTS] = pts; }
	void setAction( int ligne );
	void setAction( QList<int> liste ) { ra.setAction(liste); }
	
private:
	void trace(QString txt);
	ResultatAnalyse ra;
	QStringList label;
	QList<int> dicesList;
	int resultat[DERNIERE_LIGNE + 1][2];
	int enCoursDescendante;
	int enCoursMontante;
	QList<int> enCoursLibre;
	QList<int> lignesJouables;
	QList<int> enCoursLibreHaut;
	QList<int> enCoursLibreBas;
	int numLancer;
	int m_ecartDescendante;
	int m_ecartMontante;
	int m_ecartLibre;
	int m_avancementDescendante;
	int m_avancementMontante;
	int m_joueur;
	QList<int> m_ordrePourBarrer;
	
	Figure yahtzee;
	Figure carre;
	Figure brelan;
	Figure full;
	Figure paire1;
	Figure paire2;
	Figure deuxPaires;
	Figure petiteSuite;
	Figure grandeSuite;

	int sommeDes()
		{ return dicesList[UN] + dicesList[DEUX] + dicesList[TROIS] + dicesList[QUATRE] + dicesList[CINQ]; }
	int maxCountHaut();
	int ligneMaxPointsBas();
	QList<int> lignesMaxCountHaut();

	void marquer(); // c'est le dernier tour, il faut jouer
	bool tenterFigureNonAmeliorable();
	bool tenterAmeliorer();
	QList<int> toutSauf(Figure listeAGarder, int valMin = 7);
	QList<int> toutSauf(QList<int> listeAGarder, int valMin = 7);
	QList<int> toutSauf(int valeur, int valMin = 7);
	QList<int> garder(QList<int>);
	QList<int> listeDesDontValeurEst(int val);
	int getMostFrequentValue();
	int mini( QList<int> );
	int maxi( QList<int> );
};


#endif
