#include "yahtzeeWindow.h"
#include <QTranslator>
#include <QLibraryInfo>

void myMessageOutput(QtMsgType type, const char *msg)
{
	switch (type)
	{
		case QtDebugMsg:
			fprintf(stdout, "%s\n", msg);
			break;
		case QtWarningMsg:
			fprintf(stderr, "Avertissement : %s\n", msg);
			break;
		case QtCriticalMsg:
			fprintf(stderr, "Erreur critique : %s\n", msg);
			break;
		case QtFatalMsg:
			fprintf(stderr, "Erreur fatale : %s\n", msg);
			abort();
	}
}

QString getenv(QString var)
{
	if (!var.endsWith("=")) var += "=";
	
	QStringList env( QProcess::systemEnvironment() );	// récupère l'environnement sous forme de liste de chaînes
	// puis le parcourt
	foreach ( QString chaine, env ) {
		if ( chaine.startsWith(var) ) {	// la chaîne d'environnement débute par "<var>="
			QString formattedEnv( chaine.mid(var.size()) );	// prend tout ce qui est après le signe '='
			if (!formattedEnv.endsWith("/")) {
				formattedEnv += "/";
			}
			return formattedEnv;
		}
	}
	
	return QString();
};


int main( int argc, char *argv[] )
{

    //	***************************	//
    //	Initialisations importantes	//
    //	***************************	//

    qInstallMsgHandler(myMessageOutput);
	QApplication::setDesktopSettingsAware(true);
	QStyle* style = QApplication::setStyle("cleanlooks");
	QApplication::setPalette(style->standardPalette());
	QApplication app( argc, argv );

    // pour avoir les boutons des boîtes de dialogue dans la langue locale (fr par défaut)
    QTranslator qtTranslator;
    qtTranslator.load("qt_" + QLocale::system().name(),QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app.installTranslator(&qtTranslator);

    // pour avoir l'appli localisée
    QTranslator myappTranslator;
    myappTranslator.load( "yahtzee_" + QLocale::system().name(), QCoreApplication::applicationDirPath() );
    app.installTranslator(&myappTranslator);

    // pour les settings
    app.setOrganizationName( "PapaJaac" );
    app.setApplicationName( "YahQtzee" );
    /********************************************************/

    // semence aléatoire
    srand ( time(NULL) );

    // accès au formulaire créé par Qt4 Designer et lecture des "settings"
    yahtzeeWindow *dial = new yahtzeeWindow();

    // afficher le formulaire
    dial->show() ;
    dial->on_action_ToutRelancer_triggered ();

    //	on exécute le programme
    int resultat = app.exec();

    // 	et on quitte tout
    return resultat;

} // main
