#include <QtGui>
#include "renderarea.h"

DiceRenderArea::DiceRenderArea(QWidget *parent) : QWidget(parent)
{
//     setBackgroundRole(QPalette::Base);
//     setAutoFillBackground(false);
	setMinimumSize (QSize(63, 63));
	setMaximumSize (QSize(63, 63));

	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(anim()));

	checked = false;
	animation = false;
	enabled = true;
	diceValue = -1;	// affichage de '?'
	aspect = CLASSIQUE;	// points affichés

	selectionColour = VERT_CLAIR;	// couleur de sélection
	setCaro(0);	// easter egg
	setLogo( true );
	fast = false;
	m_marge = 2;
	m_scaleXY = 1.0;
	
	// liste des points à dessiner pour chaque valeur du dé
	m_pointNumList[0] << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8;
	m_pointNumList[1] << 4;
	m_pointNumList[2] << 2 << 6;
	m_pointNumList[3] << 2 << 4 << 6;
	m_pointNumList[4] << 0 << 2 << 6 << 8;
	m_pointNumList[5] << 0 << 2 << 4 << 6 << 8;
	m_pointNumList[6] << 0 << 2 << 3 << 5 << 6 << 8;
	m_pointNumList[7] << 0 << 2 << 4 << 6 << 7 << 8;
	
}

void DiceRenderArea::setFast(bool newFast)
{
	if (fast != newFast) {
		fast = newFast;
		update();
	} 
}

void DiceRenderArea::setLogo (bool drawLogo)
{
	m_logo = drawLogo;
	m_icon = drawLogo ? QIcon(":/images/distrologo.png") : QIcon();
}

void DiceRenderArea::setMarge(int marge)
{
	m_marge = qMin(qMax(marge,0),4);
	update();
}

void DiceRenderArea::clearSeed()
{
	m_lancer[HUMAN].clear();
	m_lancer[PC].clear();
}

void DiceRenderArea::setDiceValue (int id, int value)
{
	// donne une valeur entre 1 et 6 au dé pour l'utilisateur id.
	// ensuite redessine le dé
	// Pour une valeur aléatoire : setDiceValue(-1) ou setDiceValue()
	
	if (value != -1) {
		diceValue = value;
		update();
	}
	else {
		if (m_lancer[HUMAN].isEmpty() || m_lancer[PC].isEmpty()) {
			int valeur = alea1_6();
			m_lancer[HUMAN].enqueue(valeur);
			m_lancer[PC].enqueue(valeur);
		}
		diceValue = m_lancer[id].dequeue();
		update();
	}
}


void DiceRenderArea::setAnimation (bool anim)
{
	// anime ou immobilise le dé
	animation = anim;
	enabled = !anim;

	if (animation){
		m_angleAnim = alea( 90 );
		m_sensAnim = (alea(2) == 1 ? -1 : 1);
		m_scaleXY = (50.0 + alea(20))/100.0;
		timer->start(1000 / IMAGES_PER_SECOND);	// 125 = 8, 50 = 20 tics par seconde pour afficher le clignotement
	}
	else {
		timer->stop();
	}
};

void DiceRenderArea::anim()
{
	diceValue = alea1_6();
	repaint();
}


void DiceRenderArea::paintEvent(QPaintEvent *)
{

	// initialisations du QPainter
	QImage image(size(), QImage::Format_ARGB32);
	image.fill(0);	// fond transparent
	QPainter dicePainter(&image);
// 	QPainter dicePainter(this);

	diceDraw(dicePainter);
// 	if (!animation && diceValue == 5) image.save("/home/gilles/Bureau/yahtzee.png");

	QPainter widgetPainter(this);
	widgetPainter.drawImage(0, 0, image);
	
}

void DiceRenderArea::diceDraw(QPainter &painter)
{
// 	painter.initFrom(this);
// 	painter.setBackgroundMode(Qt::TransparentMode);
// 	painter.eraseRect( rect() );
	
	painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing, true);

	if (animation && !fast) {
		int translateX = rect().center().x();
		int translateY = rect().center().y();

		painter.translate(translateX, translateY);

		m_angleAnim += (360 / IMAGES_PER_SECOND);
		painter.rotate(m_sensAnim * m_angleAnim);
#define M_PI 3.14159265358979323846
		
		painter.scale(m_scaleXY, m_scaleXY);
		
		painter.translate(-translateX, -translateY);
	}
	
	QColor faceColour(checked ? selectionColour: Qt::white);

	if (!fast) {
		QLinearGradient linearGradient (rect().topLeft(), rect().bottomRight());
		linearGradient.setColorAt(0.0, faceColour);
		linearGradient.setColorAt(1.0, faceColour.darker(200));
		painter.setBrush (QBrush(linearGradient));
	}
	else {
		painter.setBrush (QBrush(faceColour.darker(125)));
	}
	
	painter.setPen (Qt::NoPen);
	painter.drawRoundedRect(rect(), 20.0, 20.0, Qt::RelativeSize);
	
	
	if (diceValue < 0 || diceValue > 7) return;

	QRectF rectF(rect());

	if (aspect == CHIFFRES) {
		if (m_logo) m_icon.paint(&painter, rect());
		
		QFont font;
		font.setPixelSize(height() / 2);
		font.setBold(true);
		painter.setFont(font);
		painter.setPen (Qt::black);
		painter.drawText(rectF, Qt::AlignCenter, QString("%1").arg(diceValue));
		return;
	}

	if (aspect == CLASSIQUE) {
		painter.setPen (Qt::NoPen);

		qreal xGauche = width() / 4.0;
		qreal yHaut = height() / 4.0;

		qreal xCoord[9] = {xGauche, 2 * xGauche, 3 * xGauche, xGauche, 2 * xGauche, 3 * xGauche, xGauche, 2 * xGauche, 3 * xGauche};
		qreal yCoord[9] = {yHaut, yHaut, yHaut, 2 * yHaut, 2 * yHaut, 2 * yHaut, 3 * yHaut, 3 * yHaut, 3 * yHaut};

		qreal radius = qMin(width(), height()) / 9.0 /*+ m_marge*/;
		qreal cote = radius * 2.0;
		
		foreach (int numPoint, m_pointNumList[diceValue]) {
			// x et y sont les coordonnées haut-gauche du carré dans lequel on va dessiner le creux du dé
			
			qreal x = xCoord[numPoint] - radius;
			qreal y = yCoord[numPoint] - radius;

			QRectF carre(x, y, cote, cote);

			switch (caro) {
				case 0:	// chaque creux a un aspect 'normal'
				{
					if (!fast) {
						QLinearGradient gradient(carre.topLeft(), carre.bottomRight());
						gradient.setColorAt(0.1, Qt::black);
						gradient.setColorAt(0.8, faceColour);
						painter.setBrush(QBrush(gradient));
					}
					else {
						painter.setBrush(QBrush(faceColour.darker(200)));
					}
	
					painter.drawEllipse(carre);
					if (m_logo) m_icon.paint(&painter, QRect(x, y, cote, cote));
				}
				break;

				case 1:	// chaque creux affiche la valeur du dé
				case 2:	// chaque creux affiche les initiales de...
				{
					QFont font;
					font.setPixelSize(cote);
					font.setBold(true);
					painter.setFont (font);
					painter.setPen (QPen(Qt::black));
					switch (caro) {
						case 1:
							painter.drawText(carre, Qt::AlignCenter, QString("%1").arg(diceValue));
							break;

						case 2:
							painter.drawText(carre, Qt::AlignCenter, txtCaro);
							break;
					}
				}
				break;
			}
		}
	}
}

void DiceRenderArea::mouseReleaseEvent (QMouseEvent *event)
{
	if (isEnabled())
	{
		if (event->button() == Qt::LeftButton) setChecked (!checked);

		if (event->button() == Qt::RightButton)
		{
			caro = (caro + 1) % 3;
			update();
		}
	}
};


void DiceRenderArea::setChecked (bool newCheck)
{
	if ( checked != newCheck )
	{
		checked = newCheck;
		update();
		emit diceChecked();
	}
}


void DiceRenderArea::setSelectionColour(QColor value)
{
	selectionColour = value;
	update();
};


void DiceRenderArea::setSurfaceAspect(int value)
{
	aspect = value;
	update();
};


void DiceRenderArea::setEnabled(bool enable)
{
	enabled = enable;
};

bool DiceRenderArea::isEnabled()
{
	return enabled;
};
