#include <QtGui>

#include "renderarea.h"
#include "yahtzeeWindow.h"
#include "piste.h"

#define DERNIER_COUP_ROBOT	QRgb(0x000080)


yahtzeeWindow::yahtzeeWindow()
{
	setupUi( this );

	toolBar->addAction(action_New);
	toolBar->addAction(action_Open);
	toolBar->addAction(action_Save);
	toolBar->addSeparator();
	toolBar->addAction(action_ToutRelancer);
	toolBar->addAction(action_RelancerSelection);
	toolBar->addSeparator();
	toolBar->addAction(action_Prefere);
	toolBar->addAction(action_Scores);
	toolBar->addAction(action_Logs);
	toolBar->addSeparator();
	toolBar->addAction(action_About);
	toolBar->addSeparator();
	toolBar->addAction(action_Quitter);

	demoDice = new DiceRenderArea(this);
	optionsDiceLayout->addWidget(demoDice);
	demoDice->setDiceValue(0, 6);
	connect(optionRouge, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionVert, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionJaune, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionStyleClassique, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionStyleChiffres, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionStyleSimple, SIGNAL(clicked()), this, SLOT(changeLook()));
	connect(optionStyleLogo, SIGNAL(clicked()), this, SLOT(changeLook()));


	m_labelLigne << tr("AS") << tr("DEUX") << tr("TROIS") << tr("QUATRE") << tr("CINQ") << tr("SIX") << "**" << "**";
	m_labelLigne << tr("UNE PAIRE") << tr("DEUX PAIRES") << tr("BRELAN") << tr("CARRE") << tr("FULL");
	m_labelLigne << tr("PETITE SUITE") << tr("GRANDE SUITE") << tr("CHANCE") << tr("YAHTZEE");
	m_labelColonne << tr("DESCENDANTE") << tr("MONTANTE") << tr("LIBRE") << tr("DESCENDANTE") << tr("MONTANTE") << tr("LIBRE");

	yellowBrush = QBrush(JAUNE_CLAIR);
	greenBrush = QBrush(VERT_CLAIR);
	lightRedBrush = QBrush(ROUGE_CLAIR);
	lightBlueBrush = QBrush(BLEU_CLAIR);
	whiteBrush = QBrush(Qt::white);
	redBrush = QBrush(Qt::red);
	blackBrush = QBrush(Qt::black);

// 	optionTrier->setChecked(false);

/*	columnTitles << USERNAME << USERNAME << USERNAME;
	columnTitles << tr("Robot") << tr("Robot") << tr("Robot");*/
	
	lignesCliquables << UN << DEUX <<TROIS << QUATRE << CINQ << SIX;
	lignesCliquables << UNE_PAIRE << DEUX_PAIRES << BRELAN << CARRE << FULL << PETITE_SUITE << GRANDE_SUITE << CHANCE << YAHTZEE;

	m_piste = new Piste(this);
	diceLayout = new QHBoxLayout(m_piste);
	diceLayout->setSpacing(6);
	diceLayout->setObjectName(QString::fromUtf8("diceLayout"));
	diceLayout->setSizeConstraint(QLayout::SetNoConstraint);

	QStringList tousLesStyles(QStyleFactory::keys());
	styleCombo->addItems(tousLesStyles);
	styleCombo->setCurrentIndex(-1);
	
	pisteLayout->addWidget(m_piste);
	
	// 	création des dés
	for ( int numDice = UN ; numDice <= CINQ ; numDice++ )
	{

		affDices[numDice] = new DiceRenderArea(this);
		diceLayout->addWidget( affDices[numDice] );
		affDices[numDice]->setSelectionColour(getDiceSelectionColour());

		connect(affDices[numDice], SIGNAL(diceChecked()), this, SLOT(activerDesactiverRelancerSelection()));
	}

	// 	création des cellules
	for ( int ligne = UN ; ligne <= DERNIERE_LIGNE ; ++ligne )
	{
		for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++)
		{
			witem[ligne][colonne] = new QTableWidgetItem;
			feuilleDeMarque->setItem(ligne, colonne, witem[ligne][colonne]);
		}
	}

	columnTitles << "(-000)" << "(-000)" << "(-000)";
	columnTitles << "(-000)" << "(-000)" << "(-000)";
	for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++) {
		feuilleDeMarque->horizontalHeaderItem(colonne)->setText(columnTitles[colonne]);
	}
	feuilleDeMarque->resizeColumnsToContents();

	columnTitles.clear();
	columnTitles << "" << "" << "";
	columnTitles << "" << "" << "";
	
	feuilleDeMarque->setIconSize (QSize(20,20));
	feuilleDeMarque->resizeRowsToContents();

	m_joueurEnCours = HUMAN;
	
	setup();
	readSettings();
	remplirScores(false);
	openLastGame();

};

yahtzeeWindow::~yahtzeeWindow() { close(); }

void yahtzeeWindow::trace (QColor color, QString txt) {
	logs->setTextColor(color);
	logs->append(txt);
	logs->setTextColor(Qt::black);
}


void yahtzeeWindow::close()
{
	writeSettings();
	writeLastGame();
	QMainWindow::close();
}


void yahtzeeWindow::clean()
{
	for ( int ligne = UN ; ligne <= DERNIERE_LIGNE ; ++ligne )
	{
		for ( int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++ )
		{
			witem[ligne][colonne]->setText(QString());
			witem[ligne][colonne]->setFlags(Qt::ItemIsEnabled);
		}
	}

	logs->clear();	// effacer les traces
}


void yahtzeeWindow::setup() {

	lancerEnCours = 0;
	tourEnCours = 0;
	m_joueurEnCours = HUMAN;
	ligneDescendante[HUMAN] = PREMIERE_LIGNE;
	ligneMontante[HUMAN] = DERNIERE_FIGURE;
	ligneDescendante[PC] = PREMIERE_LIGNE;
	ligneMontante[PC] = DERNIERE_FIGURE;
	lignesLibres[HUMAN].clear();
	lignesLibres[PC].clear();
// 	lastRow = -1;
// 	lastColumn = -1;
// 	lastTurn = -1;

	action_ToutRelancer->setEnabled (true);
	toutRelancer->setEnabled (true);
	action_RelancerSelection->setEnabled (false);
	relancerSelection->setEnabled (false);
	feuilleDeMarque->setEnabled (false);
// 	/*action_Annuler*/->setEnabled(false);

	demoDice->setChecked(true);
	demoDice->setDiceValue(0, 6);
	demoDice->setSelectionColour(getDiceSelectionColour());

	drawStandardGrille();
}

int yahtzeeWindow::dicesSum()
{
	int sum = affDices[UN]->getDiceValue();
	for (int numDice = DEUX ; numDice <= CINQ ; numDice++) {
		sum += affDices[numDice]->getDiceValue();
	}

	return sum;
}

void yahtzeeWindow::drawStandardGrille()
{

	int total[6];
	QFont boldFont;
	boldFont.setBold(true);

	// couleur des horizontalHeaderItem
	for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++)
	{
		feuilleDeMarque->horizontalHeaderItem(colonne)->setForeground(blackBrush);
	}

	// centrage et police des verticalHeaderItem
	for ( int ligne = UN ; ligne <= TOTAL_GENERAL ; ligne++ )
	{
		feuilleDeMarque->verticalHeaderItem(ligne)->setForeground(blackBrush);
		feuilleDeMarque->verticalHeaderItem(ligne)->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
	}

	// mise en forme des lignes de totaux
	QList<int> lignesTotaux;
	lignesTotaux << TOTAL_HAUT << BONUS << TOTAL_COLONNE << TOTAL_GENERAL;

	foreach (int ligne, lignesTotaux)
	{
		feuilleDeMarque->verticalHeaderItem(ligne)->setFont(boldFont);
		feuilleDeMarque->verticalHeaderItem(ligne)->setForeground(Qt::darkGreen);
		feuilleDeMarque->verticalHeaderItem(ligne)->setBackground(QBrush(0xF6F6F6));

		for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++)
		{
			witem[ligne][colonne]->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			witem[ligne][colonne]->setForeground(Qt::darkGreen);
			witem[ligne][colonne]->setBackground(QBrush(0xF6F6F6));
			witem[ligne][colonne]->setFont(boldFont);
		}
	}

	// aspect des cellules et calcul des totaux
	for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++)
	{
		total[colonne] = 0;
		int sommeNumeros = 0;
		int sommeValeurs = 0;

		for ( int ligne = UN ; ligne <= SIX ; ++ligne ) {
			witem[ligne][colonne]->setFlags(Qt::ItemIsEnabled);
			QString cellText(witem[ligne][colonne]->text());
			if (!cellText.isEmpty()) {
				int numero = ligne + 1;
				sommeNumeros += numero;

				int valeur = cellText.toInt();
				sommeValeurs += valeur;

				int ecart = valeur - (3 * numero);
				if (colonne >= DESCENDANTE_PC && colonne <= LIBRE_PC) {
					witem[ligne][colonne]->setBackground(lightBlueBrush);
				}
				else {
					witem[ligne][colonne]->setBackground(ecart >= 0 ? greenBrush : lightRedBrush);
				}
			}
			else {
				witem[ligne][colonne]->setBackground(whiteBrush);
			}
		}

		int ecart = sommeValeurs - (3 * sommeNumeros);
		if (ecart > 0) {
			feuilleDeMarque->horizontalHeaderItem(colonne)->setText(columnTitles[colonne] + QString(" (+%1)").arg(ecart));
		}
		else {
			if (ecart < 0) {
				feuilleDeMarque->horizontalHeaderItem(colonne)->setText(columnTitles[colonne] + QString(" (%1)").arg(ecart));
			}
			else {
				feuilleDeMarque->horizontalHeaderItem(colonne)->setText(columnTitles[colonne]);
			}
		}

		switch (colonne) {
			case DESCENDANTE_HUMAN:
				m_ecartDesc[HUMAN] = ecart;
				break;
			case MONTANTE_HUMAN:
				m_ecartAsc[HUMAN] = ecart;
				break;
			case LIBRE_HUMAN:
				m_ecartLibre[HUMAN] = ecart;
				break;
			case DESCENDANTE_PC:
				m_ecartDesc[PC] = ecart;
				break;
			case MONTANTE_PC:
				m_ecartAsc[PC] = ecart;
				break;
			case LIBRE_PC:
				m_ecartLibre[PC] = ecart;
				break;
		}


		
		for ( int ligne = UN ; ligne <= SIX ; ++ligne ) {
			witem[ligne][colonne]->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			witem[ligne][colonne]->setForeground(blackBrush);
			QString cellText(witem[ligne][colonne]->text());
			total[colonne] += witem[ligne][colonne]->text().toInt();
		}

		witem[TOTAL_HAUT][colonne]->setText(QString("%1").arg(total[colonne]));

		int bonus = 0;
		if (total[colonne] >= 63) bonus = 35;

		total[colonne] += bonus;

		witem[BONUS][colonne]->setText(QString("%1").arg(bonus));

		for ( int ligne = PREMIERE_FIGURE ; ligne <= YAHTZEE ; ++ligne )
		{
			witem[ligne][colonne]->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			witem[ligne][colonne]->setForeground(blackBrush);
			witem[ligne][colonne]->setFlags(Qt::ItemIsEnabled);
			
			// couleur du fond
			QString cellText(witem[ligne][colonne]->text());
			if (cellText.isEmpty())
			{
				witem[ligne][colonne]->setBackground(whiteBrush);
			}
			else {
				if (colonne >= DESCENDANTE_PC && colonne <= LIBRE_PC) {
					witem[ligne][colonne]->setBackground(lightBlueBrush);
				}
				else {
					if (cellText.toInt() > 0) {
						witem[ligne][colonne]->setBackground(greenBrush);
					}
					else {
						witem[ligne][colonne]->setBackground(lightRedBrush);
					}
				}
				total[colonne] += cellText.toInt();
			}
		}
		witem[TOTAL_COLONNE][colonne]->setText(QString("%1").arg(total[colonne]));
	}

	witem[TOTAL_GENERAL][MONTANTE_HUMAN]->setText(QString("%1").arg(total[DESCENDANTE_HUMAN] + total[MONTANTE_HUMAN] + total[LIBRE_HUMAN]));
	witem[TOTAL_GENERAL][MONTANTE_PC]->setText(QString("%1").arg(total[DESCENDANTE_PC] + total[MONTANTE_PC] + total[LIBRE_PC]));
	if (witem[TOTAL_GENERAL][MONTANTE_HUMAN]->text().toInt() > witem[TOTAL_GENERAL][MONTANTE_PC]->text().toInt()) {
		witem[TOTAL_GENERAL][MONTANTE_HUMAN]->setIcon(QIcon(":/images/flag-green.png"));
		witem[TOTAL_GENERAL][MONTANTE_PC]->setIcon(QIcon(":/images/flag-red.png"));
	}
	else {
		witem[TOTAL_GENERAL][MONTANTE_HUMAN]->setIcon(QIcon(":/images/flag-red.png"));
		witem[TOTAL_GENERAL][MONTANTE_PC]->setIcon(QIcon(":/images/flag-green.png"));
	}
// 	if (witem[TOTAL_GENERAL][MONTANTE_HUMAN]->text().toInt() > witem[TOTAL_GENERAL][MONTANTE_PC]->text().toInt()) {
// 		witem[TOTAL_GENERAL][MONTANTE_HUMAN]->setBackground(greenBrush);
// 		witem[TOTAL_GENERAL][MONTANTE_PC]->setBackground(lightRedBrush);
// 	}
// 	else {
// 		witem[TOTAL_GENERAL][MONTANTE_PC]->setBackground(greenBrush);
// 		witem[TOTAL_GENERAL][MONTANTE_HUMAN]->setBackground(lightRedBrush);
// 	}

// 	feuilleDeMarque->resizeColumnsToContents();

	// calcul des lignes libres
	lignesLibres[HUMAN].clear();
	lignesLibres[PC].clear();

	foreach (int ligne, lignesCliquables) {
		if (witem[ligne][LIBRE_HUMAN]->text().isEmpty()) {
			lignesLibres[HUMAN] << ligne;
		}
		if (witem[ligne][LIBRE_PC]->text().isEmpty()) {
			lignesLibres[PC] << ligne;
		}
	}

	feuilleDeMarque->setToolTip ( tr("les dés totalisent %1 points").arg(dicesSum()) );

}

void yahtzeeWindow::enregistrerScores() {
	
	// bloquer les boutons
	action_ToutRelancer->setEnabled (false);
	toutRelancer->setEnabled (false);
	action_RelancerSelection->setEnabled (false);
	relancerSelection->setEnabled (false);
	feuilleDeMarque->setEnabled (false);

	// Afficher "j'ai gagné" ou "vous avez gagné"
	int scoreHumain = witem[TOTAL_GENERAL][MONTANTE_HUMAN]->text().toInt();
	int scorePC = witem[TOTAL_GENERAL][MONTANTE_PC]->text().toInt();
	QString felicitations;
	int vainqueur;
	QString nomVainqueur;
	int meilleurScore = scoreHumain;
	QString titre (tr("Fin de partie"));
	
	if (scoreHumain > scorePC) {
		felicitations = tr("Félicitations, vous avez gagné !\nEnregistrer le score sous le nom :");
		nomVainqueur = QInputDialog::getText ( this, titre, felicitations, QLineEdit::Normal, USERNAME);
		vainqueur = HUMAN;
		meilleurScore = scoreHumain;
	}

	if (scoreHumain < scorePC) {
		felicitations = tr("J'ai gagné !");
		QMessageBox::information(this, titre, felicitations);
		vainqueur = PC;
		nomVainqueur = "PC";
		meilleurScore = scorePC;
	}

	if (scoreHumain == scorePC) {
		felicitations = tr("Nous sommes ex-aequo !");
		QMessageBox::information(this, titre, felicitations);
		nomVainqueur = USERNAME;
		meilleurScore = scoreHumain;
	}

// 	if (nomVainqueur.isEmpty()) return;

	// insérer le nouveau score
	m_scores.date << QDate::currentDate().toString(Qt::DefaultLocaleShortDate) ;
	m_scores.auteur << nomVainqueur;
	m_scores.score << QString("%1").arg(meilleurScore);
	remplirScores(false);
}




void yahtzeeWindow::on_action_New_triggered()
{

	if ( (jeuEnCours() && OKPourContinuer()) || jeuNonCommence() || jeuTermine() ) {
		clean();
		setup();
		trace (Qt::red, tr("Nouvelle partie"));
		on_action_ToutRelancer_triggered();
	}
}

void yahtzeeWindow::on_action_ToutRelancer_triggered()
{

	// désactive save / il ne sera réactivé qu'à la fin du tout du PC
	action_Save->setEnabled(false);
	
	checkAllDices();
	on_action_RelancerSelection_triggered ();
}

void yahtzeeWindow::on_action_RelancerSelection_triggered ()
{
// 	action_Annuler->setEnabled(false);
	drawStandardGrille();

// 	Incrémenter le n° de lancer et l'afficher
	lancerEnCours %= 3;
	lancerEnCours += 1;

	if (lancerEnCours == 1) {
		for ( int numDice = UN ; numDice <= CINQ ; numDice++ )
			affDices[numDice]->clearSeed();
	}


	doRoll();
	finDeLancer();
	

}


void yahtzeeWindow::doRoll() {
	
// Bloquer les boutons
	action_ToutRelancer->setEnabled (false);
	toutRelancer->setEnabled (false);
	action_RelancerSelection->setEnabled (false);
	relancerSelection->setEnabled (false);
	feuilleDeMarque->setEnabled (false);

// Pointeur d'attente

	if (0 != delaiAttente()) {
		// 	Lancer les animations individuelles
		disableAllDices();
		QApplication::setOverrideCursor(Qt::WaitCursor);
		for ( int numDice = UN ; numDice <= CINQ ; numDice++ ) {
			if (affDices[numDice]->isChecked()) affDices[numDice]->setAnimation(true);
		}
	}
	// on attend x ms avant de débloquer la situation : finDeLancer()
	enCours->setTitle(tr("Lancer en cours..."));
	QTimer::singleShot(delaiAttente(), this, SLOT(rolling()));

	endRoll = false;
	while (!endRoll) {
		qApp->processEvents();
	}

	for ( int numDice = UN ; numDice <= CINQ ; ++numDice ) {
		if ( affDices[numDice]->isChecked () ) {
			affDices[numDice]->setAnimation( false );
		}
	}


	if (lancerEnCours < 3) {
		activerDesactiverRelancerSelection();
		toutRelancer->setEnabled (true);
		enableAllDices();
	}
	else {
		disableAllDices();
	}

	QApplication::restoreOverrideCursor();

};

void yahtzeeWindow::rolling()
{
	endRoll = true;
}

int yahtzeeWindow::delaiAttente() {
	if (optionSameDelay->isChecked()) {
		return optionDelai->value() * 1000;
	}
	else {
		if (m_joueurEnCours == HUMAN) return optionDelai->value() * 1000; else return 0;
	}
}

void yahtzeeWindow::activerDesactiverRelancerSelection()
{

	bool enableRelancer = (lancerEnCours > 0 && lancerEnCours < 3);
	bool enable = affDices[UN]->isChecked();
	enable |= affDices[DEUX]->isChecked();
	enable |= affDices[TROIS]->isChecked();
	enable |= affDices[QUATRE]->isChecked();
	enable |= affDices[CINQ]->isChecked();
	enable &= enableRelancer ;

	action_ToutRelancer->setEnabled(enableRelancer);
	action_RelancerSelection->setEnabled(enable);
	relancerSelection->setEnabled(enable);

}

void yahtzeeWindow::finDeLancer()
{
		// attribuer des valeurs
	for ( int numDice = UN ; numDice <= CINQ ; ++numDice ) {
		if ( affDices[numDice]->isChecked ()) {
			affDices[numDice]->setDiceValue (m_joueurEnCours);
		}
	}

	int d1, d2, d3, d4, d5;
	d1 = affDices[UN]->getDiceValue();
	d2 = affDices[DEUX]->getDiceValue();
	d3 = affDices[TROIS]->getDiceValue();
	d4 = affDices[QUATRE]->getDiceValue();
	d5 = affDices[CINQ]->getDiceValue();
	
	trace (Qt::black, tr("Lancer n°%1 de %2 : %3 %4 %5 %6 %7").arg(lancerEnCours).arg(nomDeJoueur()).arg(d1).arg(d2).arg(d3).arg(d4).arg(d5));
	uncheckAllDices();
	
	showEnCours();
	afficherAnalyse ();
	
}

void yahtzeeWindow::afficherAnalyse ()
{
	drawStandardGrille();

	analyse.init(m_joueurEnCours, lancerEnCours, ligneDescendante[m_joueurEnCours], ligneMontante[m_joueurEnCours], lignesLibres[m_joueurEnCours],
					affDices[UN]->getDiceValue(),
					affDices[DEUX]->getDiceValue(),
					affDices[TROIS]->getDiceValue(),
					affDices[QUATRE]->getDiceValue(),
					affDices[CINQ]->getDiceValue(),
					m_ecartDesc[m_joueurEnCours],
					m_ecartAsc[m_joueurEnCours],
					m_ecartLibre[m_joueurEnCours]);

	possibleList = analyse.getPossible();
	ra = analyse.getResultatAnalyse();

	if (optionSuggestion->isChecked()) {
		if (ra.getAction() == ResultatAnalyse::relancer) {
			foreach(int numDice, ra.getRejouer()) {
				affDices[numDice]->setChecked( true );
			}
		}
	}

	if (m_joueurEnCours == HUMAN) {
		foreach (int ligne, possibleList) {
			if (ligne == ligneDescendante[m_joueurEnCours]) {
				witem[ligne][m_joueurEnCours == HUMAN ? DESCENDANTE_HUMAN : DESCENDANTE_PC]->setBackground(yellowBrush);
			}
			if (ligne == ligneMontante[m_joueurEnCours]) {
				witem[ligne][m_joueurEnCours == HUMAN ? MONTANTE_HUMAN : MONTANTE_PC]->setBackground(yellowBrush);
			}
			if (lignesLibres[m_joueurEnCours].contains(ligne)) {
				witem[ligne][m_joueurEnCours == HUMAN ? LIBRE_HUMAN : LIBRE_PC]->setBackground(yellowBrush);
			}
		}
	}

	feuilleDeMarque->setEnabled (true);
};

void yahtzeeWindow::on_feuilleDeMarque_cellClicked(int ligneCliquee, int colonneCliquee)
{
// 	if (!lignesCliquables.contains(ligneCliquee)) return;

	bool changed = false;
// 	lastRow = ligneCliquee;
// 	lastColumn = colonneCliquee;
// 	lastTurn = lancerEnCours;
// 	lastDown = ligneDescendante[m_joueurEnCours];
// 	lastUp = ligneMontante[m_joueurEnCours];
	if (m_joueurEnCours == HUMAN && colonneCliquee > LIBRE_HUMAN) return;

	switch (colonneCliquee) {
		case DESCENDANTE_HUMAN:
		case DESCENDANTE_PC:
// 			vérifie que clic sur la case qui va bien dans la colonne descedante
			if (ligneCliquee == ligneDescendante[m_joueurEnCours] && ligneDescendante[m_joueurEnCours] <= YAHTZEE) {
				if (ligneDescendante[m_joueurEnCours] == SIX) ligneDescendante[m_joueurEnCours] = PREMIERE_FIGURE; else ligneDescendante[m_joueurEnCours]++;
				changed = true;
			}
			break;
		case MONTANTE_HUMAN:
		case MONTANTE_PC:
			if (ligneCliquee == ligneMontante[m_joueurEnCours] && ligneMontante[m_joueurEnCours] >= UN) {
				if (ligneMontante[m_joueurEnCours] == PREMIERE_FIGURE) ligneMontante[m_joueurEnCours] = SIX; else ligneMontante[m_joueurEnCours]--;
				changed = true;
			}
			break;
		case LIBRE_HUMAN:
		case LIBRE_PC:
			if (possibleList.contains(ligneCliquee) || witem[ligneCliquee][colonneCliquee]->text().isEmpty()) {
				changed = true;
			}
			break;
	}

	if (changed)
	{
// 		action_Annuler->setEnabled(true);

		
		int points = analyse.getPoints(ligneCliquee);
		trace (Qt::blue, tr("--> %1 choisit la ligne %2 dans la colonne %3 et marque %4 point(s).")
				.arg(nomDeJoueur())
				.arg(m_labelLigne[ligneCliquee])
				.arg(m_labelColonne[colonneCliquee])
				.arg(points));

		witem[ligneCliquee][colonneCliquee]->setText(QString("%1").arg(points));

		feuilleDeMarque->setEnabled (false);
		lancerEnCours = 3;
		action_ToutRelancer->setEnabled (true);
		toutRelancer->setEnabled (true);
		uncheckAllDices();
		disableAllDices();
		drawStandardGrille();
		witem[ligneCliquee][colonneCliquee]->setText(QString("%1").arg(points));
		witem[ligneCliquee][colonneCliquee]->setForeground(Qt::white);
		witem[ligneCliquee][colonneCliquee]->setBackground(QColor(DERNIER_COUP_ROBOT));
		passerLaMain();
	}
};

void yahtzeeWindow::passerLaMain()
{
	if (m_joueurEnCours == HUMAN) {
// 		do {
		m_joueurEnCours = PC;
		lancerEnCours = 0;
		enableAllDices(true);
		checkAllDices(true);
		int action;
		
		do {
			lancerEnCours++;
			doRoll();
			finDeLancer();
			action = ra.getAction();
			if (action == ResultatAnalyse::marquerLigne) {
				int ligne = ra.getLigne();
				int colonne = ra.getColonne() + DESCENDANTE_PC;
				
				on_feuilleDeMarque_cellClicked(ligne, colonne);
			}
			else {
				if (action == ResultatAnalyse::relancer) {
					foreach(int numDice, ra.getRejouer()) {
						affDices[numDice]->setChecked( true );
					}
				}
			}
		
		} while (lancerEnCours < 3 && action == ResultatAnalyse::relancer);
// 		} while (!jeuTermine());
		
		action_Save->setEnabled(true);
		
		if (jeuTermine()) {
			enregistrerScores();
		}
	}
	
	m_joueurEnCours = HUMAN;
}


void yahtzeeWindow::showEnCours()
{
	QString texte(lancerEnCours == 0 ? tr("Début du tour...") : tr("Lancer n°%1/3 (%2)").arg(lancerEnCours).arg(nomDeJoueur()));
	enCours->setTitle(texte);
}

int yahtzeeWindow::getDiceSelectionColour()
{
	if (optionRouge->isChecked()) return ROUGE_CLAIR;

	if (optionJaune->isChecked()) return JAUNE_CLAIR;

	if (optionVert->isChecked()) return VERT_CLAIR;

	return VERT_CLAIR;
};


int yahtzeeWindow::getDiceSurfaceAspect()
{
	if (optionStyleClassique->isChecked()) return CLASSIQUE;

	if (optionStyleChiffres->isChecked()) return CHIFFRES;

	return CLASSIQUE;
};

void yahtzeeWindow::writeLastGame()
{
	if (!dernierePartie->isChecked()) return;

	QDir homeDir(QDir::homePath());
	homeDir.mkdir(".yahqtzee");
	QString dataFileName(QDir::toNativeSeparators (QDir::homePath () + "/.yahqtzee/yahqtzee.last"));
	enregistrer(dataFileName, false);	// false : pas de messages d'erreur !
}

void yahtzeeWindow::openLastGame()
{
	if (!dernierePartie->isChecked()) return;

	QDir homeDir(QDir::homePath());
	homeDir.mkdir(".yahqtzee");
	QString dataFileName(QDir::toNativeSeparators (QDir::homePath () + "/.yahqtzee/yahqtzee.last"));
	ouvrir(dataFileName, false);	// false : pas de messages d'erreur !
}

void yahtzeeWindow::writeSettings()
{
	QSettings settings(this);

	settings.beginGroup( "Affichage" );
	settings.setValue("CouleurSelection", getDiceSelectionColour());
	settings.setValue("Delai", optionDelai->value());
	settings.setValue("DelaiIdentique", optionSameDelay->isChecked());
	settings.setValue("AspectSurface", getDiceSurfaceAspect());
	settings.setValue("Suggestion", optionSuggestion->isChecked());
	settings.setValue("RechargerPartie", dernierePartie->isChecked());
	settings.setValue("AffRapide", optionStyleSimple->isChecked());
	settings.setValue("Logo", optionStyleLogo->isChecked());
	settings.setValue("Marge", 3);
	settings.setValue("QtStyle", styleCombo->currentText());
	settings.endGroup();

	settings.beginGroup( "Scores" );
	settings.setValue( "Dates", m_scores.date);
	settings.setValue( "Auteurs", m_scores.auteur);
	settings.setValue( "Scores", m_scores.score);
	settings.endGroup();
}

QString yahtzeeWindow::nomDeJoueur() {
	if (m_joueurEnCours == HUMAN) {
		return USERNAME;
	}
	else {
		return "Robot";
	}
}

void yahtzeeWindow::readSettings()
{
	QSettings settings(this);

	settings.beginGroup( "Affichage" );
	int couleur = settings.value("CouleurSelection", VERT_CLAIR).toInt();
	optionDelai->setValue(settings.value("Delai", 2).toInt());
	optionSameDelay->setChecked(settings.value("DelaiIdentique", false).toBool());
	int surface = settings.value("AspectSurface", CLASSIQUE).toInt();
	optionSuggestion->setChecked(settings.value("Suggestion", false).toBool());
	dernierePartie->setChecked(settings.value("RechargerPartie", true).toBool());
	optionStyleSimple->setChecked(settings.value("AffRapide", false).toBool());
	optionStyleLogo->setChecked(settings.value("Logo", false).toBool());
	m_marge = settings.value("Marge", 2).toInt();
	QApplication::setStyle(settings.value("QtStyle", "cleanlooks").toString());
	settings.endGroup();

	// marge
	for (int numDice = UN ; numDice <= CINQ ; numDice++) affDices[numDice]->setMarge(m_marge);
	
	switch (couleur)
	{
		case VERT_CLAIR:
			optionVert->setChecked(true);
			break;
		case JAUNE_CLAIR:
			optionJaune->setChecked(true);
			break;
		case ROUGE_CLAIR:
			optionRouge->setChecked(true);
			break;
		default:
			optionVert->setChecked(true);
			break;
	}

	switch (surface)
	{
		case CLASSIQUE:
			optionStyleClassique->setChecked(true);
			break;
		case CHIFFRES:
			optionStyleChiffres->setChecked(true);
			break;
		default:
			optionStyleClassique->setChecked(true);
			break;
	}
	changeLook();

	QStringList vide;

	settings.beginGroup( "Scores" );
	m_scores.date = settings.value("Dates", vide).toStringList();
	m_scores.auteur = settings.value("Auteurs", vide).toStringList();
	m_scores.score = settings.value("Scores", vide).toStringList();
	settings.endGroup();
}


void yahtzeeWindow::changeLook()
{
	demoDice->setSelectionColour(QColor(getDiceSelectionColour()));
	demoDice->setSurfaceAspect(getDiceSurfaceAspect());
	demoDice->setFast(optionStyleSimple->isChecked());
	demoDice->setLogo(optionStyleLogo->isChecked());
	for ( int numDice = UN ; numDice <= CINQ ; numDice++ ) {
		affDices[numDice]->setSelectionColour(QColor(getDiceSelectionColour()));
		affDices[numDice]->setSurfaceAspect(getDiceSurfaceAspect());
		affDices[numDice]->setFast(optionStyleSimple->isChecked());
		affDices[numDice]->setLogo(optionStyleLogo->isChecked());
	}
}

bool yahtzeeWindow::OKPourContinuer()
{

	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("Confirmation"));
	msgBox.setIcon(QMessageBox::Warning);
	msgBox.setText(tr("Cette fonction efface la feuille de score et lance une nouvelle partie."));
	msgBox.setInformativeText(tr("Votre score ne sera pas enregistré."));
	msgBox.addButton(QMessageBox::Cancel);
	QPushButton* continueButton = msgBox.addButton(tr("&Continuer"), QMessageBox::AcceptRole);
	continueButton->setIcon(QIcon(":/images/filenew.png"));
	msgBox.exec();
	return ( msgBox.clickedButton() == continueButton );
}


bool yahtzeeWindow::jeuTermine()
{
	return lignesLibres[PC].isEmpty() && ligneDescendante[PC] > YAHTZEE && ligneMontante[PC] < UN;
}

bool yahtzeeWindow::jeuNonCommence()
{
	return DERNIERE_FIGURE-1 == lignesLibres[PC].count() && UN == ligneDescendante[PC] && YAHTZEE == ligneMontante[PC];
}

bool yahtzeeWindow::jeuEnCours()
{
	return !jeuTermine() && ! jeuNonCommence();
}



void yahtzeeWindow::enregistrer(QString &fileName, bool msg)
{
	QFile file(fileName);

	if (!file.open(QIODevice::WriteOnly))
	{
		if (msg) QMessageBox::warning(this,tr("YahQtzee"), tr("Impossible d'ecrire dans le fichier %1").arg(file.fileName()));
		return;
	}

	QDataStream out(&file);

// 	out.setVersion(*******);



	// 1/ sauver MAGIC
	out << MAGIC2;

	QApplication::setOverrideCursor(Qt::WaitCursor);

	// 2/ sauver les valeurs des dés
// 	for (int numDice = UN ; numDice <= CINQ ; numDice++)
// 	{
// 		out << affDices[numDice]->getDiceValue();
// 	}

	// 3/ sauver le n° de lancer et le joueur en cours
// 	out << lancerEnCours;
// 	out << m_joueurEnCours;

	// 4/ sauver les n° de lignes montante et descedante
	out << ligneDescendante[HUMAN] << ligneMontante[HUMAN];
	out << ligneDescendante[PC] << ligneMontante[PC];

	// 5/ sauver la feuilleDeMarque
	
	for (int ligne = UN ; ligne <= YAHTZEE ; ligne++) {
		for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++) {
			QString contenu = witem[ligne][colonne]->text();
			out << contenu;
		}
	}

	QString logsHtml(logs->toHtml());
	out << logsHtml;

	QApplication::restoreOverrideCursor();

}

void yahtzeeWindow::ouvrir(QString &fileName, bool msg)
{
	QFile file(fileName);

	if (!file.open(QIODevice::ReadOnly))
	{
		if (msg) QMessageBox::warning(this,tr("YahQtzee"), tr("Impossible d'ouvrir le fichier %1.").arg(file.fileName()));
		return;
	}

	QDataStream in(&file);

// 	in.setVersion(*******);


	// 1/ vérifier MAGIC
	int magic;
	in >> magic;
	if (magic != MAGIC1 && magic != MAGIC2)
	{
		QMessageBox::warning(this,tr("YahQtzee"), tr("Le fichier %1 n'est pas lisible par YahQtzee.").arg(file.fileName()));
		return;
	}

	QApplication::setOverrideCursor(Qt::WaitCursor);

	clean();
	drawStandardGrille();

	lancerEnCours = 0;

	// 4/ restaurer les n° de lignes montante et descedante
	in >> ligneDescendante[HUMAN] >> ligneMontante[HUMAN];
	in >> ligneDescendante[PC] >> ligneMontante[PC];

	int l;
	int c;
	QString contenu;
	
	switch (magic) {
		case MAGIC1:
			// 5/ restaurer la feuilleDeMarque
			while (!in.atEnd())
			{
				in >> l >> c >> contenu;
				witem[l][c]->setText(contenu);
			}
			break;

		case MAGIC2:
			// 5/ restaurer la feuilleDeMarque
			for (int ligne = UN ; ligne <= YAHTZEE ; ligne++) {
				for (int colonne = DESCENDANTE_HUMAN ; colonne <= LIBRE_PC ; colonne++) {
					in >> contenu;
					witem[ligne][colonne]->setText(contenu);
				}
			}
			
			QString logsHtml;
			in >> logsHtml;
			logs->setHtml(logsHtml);
	
			break;
	}
	
	QApplication::restoreOverrideCursor();

	drawStandardGrille();
}

void yahtzeeWindow::on_action_Save_triggered()
{
	QString fileName(QFileDialog::getSaveFileName(this, tr("Enregistrer la feuille de score"), QDir::homePath(), tr("Feuille de score (*.fsy)")));
	if (!fileName.isEmpty())
	{
		enregistrer(fileName);
	}
}

void yahtzeeWindow::on_action_Open_triggered()
{
	QString fileName( QFileDialog::getOpenFileName(this, tr("Ouvrir une feuille de score"), QDir::homePath(), tr("Feuille de score (*.fsy);;Tous (*)")));

	if (!fileName.isEmpty())
	{
		ouvrir(fileName);
	}

}

void yahtzeeWindow::enableAllDices(bool enable)
{
	for ( int numDice = UN ; numDice <= CINQ ; ++numDice )
	{
		affDices[numDice]->setEnabled( enable );
	}
}

void yahtzeeWindow::checkAllDices(bool check)
{
	for ( int numDice = UN ; numDice <= CINQ ; ++numDice )
	{
		affDices[numDice]->setChecked( check );
	}
}

void yahtzeeWindow::remplirScores(bool modif)
{
	int nombreScores = (m_scores.date).count();

	label->setVisible(modif);

	tableScore->setRowCount(nombreScores);

	for (int ligne = 0 ; ligne < nombreScores ; ligne++)
	{
		for (int colonne = 0 ; colonne < 3 ; colonne++)
		{
			QTableWidgetItem *witem = new QTableWidgetItem;
			tableScore->setItem(ligne, colonne, witem);
			witem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			witem->setFlags(Qt::ItemIsEnabled);
			
			switch (colonne)
			{
				case 0:
					witem->setText(m_scores.date[ligne]);
					break;
				case 1:
/*					if (m_scores.auteur[ligne] == NO_USER)*/
					if (ligne == nombreScores - 1 && modif) {
						witem->setForeground(Qt::red);
						witem->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled);
					}

					witem->setText(m_scores.auteur[ligne]);
					break;
				case 2:
					witem->setText(m_scores.score[ligne]);
					break;
				default:
					break;
			}
		}
		
	}
	tableScore->resizeRowsToContents();
	tableScore->sortItems (2, Qt::DescendingOrder );
}

void yahtzeeWindow::on_action_Prefere_triggered()
{
	action_About->setChecked(false);
	action_Scores->setChecked(false);
	action_Logs->setChecked(false);
	tabWidget->setCurrentIndex(action_Prefere->isChecked() ? 1 : 0);
};

void yahtzeeWindow::on_action_Scores_triggered()
{
	action_About->setChecked(false);
	action_Prefere->setChecked(false);
	action_Logs->setChecked(false);
	tabWidget->setCurrentIndex(action_Scores->isChecked() ? 2 : 0);
};

void yahtzeeWindow::on_action_About_triggered()
{
	action_Prefere->setChecked(false);
	action_Scores->setChecked(false);
	action_Logs->setChecked(false);
	tabWidget->setCurrentIndex(action_About->isChecked() ? 3 : 0);
};

void yahtzeeWindow::on_action_Logs_triggered()
{
	action_About->setChecked(false);
	action_Prefere->setChecked(false);
	action_Scores->setChecked(false);
	tabWidget->setCurrentIndex(action_Logs->isChecked() ? 4 : 0);
};
