#ifndef	YAHTZEE_DIALOG_H
#define	YAHTZEE_DIALOG_H

#include "ui_yahtzee.h"
#include "analyse.h"
#include "renderarea.h"
#include <QLabel>
#include <QString>
#include <QMessageBox>
#include "piste.h"
#include "figures.h"


#define MAX_ROULE 				50
#define TOUT_DESSINER			5
#define DESCENDANTE_HUMAN		0
#define MONTANTE_HUMAN			1
#define LIBRE_HUMAN				2
#define DESCENDANTE_PC			3
#define MONTANTE_PC				4
#define LIBRE_PC				5

extern QString getenv(QString var);

#define MAGIC1	0x7591
#define MAGIC2	0x7592
#define USERNAME getenv("USER")

class Scores
{
	public:
		Scores(){};
		~Scores(){};
		void clear() { date.clear(); auteur.clear(); score.clear(); }

		QStringList date;
		QStringList auteur;
		QStringList score;
};



class yahtzeeWindow : public QMainWindow, public Ui::yahtzeeDialog
{
	Q_OBJECT

	public:
		yahtzeeWindow();
		~yahtzeeWindow();

	private:
		QStringList m_labelLigne;
		QStringList m_labelColonne;
		QString nomDeJoueur();

		Piste* m_piste;
		Scores m_scores;
		bool endRoll;
		QHBoxLayout* diceLayout;
		int m_joueurEnCours;
		QStringList columnTitles;
		DiceRenderArea *affDices[5];
		DiceRenderArea *demoDice;
		QTableWidgetItem *witem[DERNIERE_LIGNE - PREMIERE_LIGNE + 1][6];
		int lancerEnCours;
		int tourEnCours;
		int ligneDescendante[2];
		int ligneMontante[2];
		QList<int> lignesLibres[2];
		QList<int> lignesCliquables;
		QList<int> possibleList;
		QList<int> suggestionList;
/*		int lastRow;
		int lastColumn;
		int lastTurn;
		int lastDown;
		int lastUp;*/
		QBrush yellowBrush;
		QBrush greenBrush;
		QBrush redBrush;
		QBrush lightRedBrush;
		QBrush blackBrush;
		QBrush whiteBrush;
		QBrush lightBlueBrush;
		Analyse analyse;
		ResultatAnalyse ra;
		int m_marge;
		QTextStream m_traces;
		int m_ecartDesc[2];
		int m_ecartAsc[2];
		int m_ecartLibre[2];
		
		int dicesSum();

		void trace (QColor, QString);
// 		QTextStream& trace() { return m_traces; }
		void remplirScores(bool);
		int delaiAttente();
		void afficherAnalyse ();
		void drawStandardGrille();
		void setup();
		void clean();
		void showEnCours();
		int getDiceSurfaceAspect();
		int getDiceSelectionColour();
		void enableAllDices(bool enable = true);
		void checkAllDices(bool check = true);
		void disableAllDices() { enableAllDices( false ); };
		void uncheckAllDices() { checkAllDices( false ); };
		void passerLaMain();

		void doRoll();

		bool OKPourContinuer();
		bool jeuTermine();
		bool jeuNonCommence();
		bool jeuEnCours();
		void ouvrir(QString &, bool msg = true);
		void enregistrer(QString &, bool msg = true);
		void enregistrerScores();

		void readSettings();
		void writeSettings();
		void writeLastGame();
		void openLastGame();

	public slots:
		void on_action_ToutRelancer_triggered ();
		void close();

	private slots:
		void rolling();
		void on_action_RelancerSelection_triggered ();

		void activerDesactiverRelancerSelection ();
		void finDeLancer();
		void on_feuilleDeMarque_cellClicked(int row, int column);
		void on_styleCombo_activated(QString style) { QApplication::setStyle(style); };
// 		void on_action_Annuler_triggered();
		void on_action_New_triggered();
		void on_action_Prefere_triggered();
		void on_action_Scores_triggered();
		void on_action_About_triggered();
		void on_action_Open_triggered();
		void on_action_Save_triggered();
		void on_action_Logs_triggered();
		void changeLook();
};

#endif
